@@ -1,6 +1,5 @@
 /* Utility and Unix shadow routines for GNU Emacs support programs on NT.
-   Copyright (C) 1994, 2002, 2003, 2004, 2005, 2006, 2007,
-                 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+   Copyright (C) 1994, 2002-2012 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -30,25 +29,19 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 #ifdef sleep
 #undef sleep
 #endif
-void sleep(unsigned long seconds);
+void sleep (unsigned long seconds);
 char *getwd (char *dir);
-int getppid(void);
-char * getlogin ();
+int getppid (void);
+char * getlogin (void);
 char * cuserid (char * s);
-unsigned getuid ();
-unsigned getegid ();
-unsigned getgid ();
+unsigned getuid (void);
+unsigned getegid (void);
+unsigned getgid (void);
 int setuid (unsigned uid);
 int setregid (unsigned rgid, unsigned gid);
 char * getpass (const char * prompt);
 int fchown (int fd, unsigned uid, unsigned gid);
 
-#ifndef BSTRING
-#define bzero(b, l) memset(b, 0, l)
-#define bcopy(s, d, l) memcpy(d, s, l)
-#define bcmp(a, b, l) memcmp(a, b, l)
-#endif
-
 /* redirect or undo interceptions created by config.h */
 #undef access
 #define access  _access
@@ -112,5 +105,3 @@ int fchown (int fd, unsigned uid, unsigned gid);
 
 /* end of ntlib.h */
 
-/* arch-tag: 93444f66-7b98-4aa5-a5cd-01444094af28
-   (do not change this comment) */
