@@ -1,7 +1,6 @@
 ;;; calc-map.el --- higher-order functions for Calc
 
-;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2012 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
 ;; Maintainer: Jay Belanger <jay.p.belanger@gmail.com>
@@ -572,7 +571,7 @@
 	     (and nargs forcenargs (/= nargs forcenargs) (>= nargs 0)
 		  (error "Must be a %d-argument operator" nargs)))
 	    ((memq key '(?\$ ?\'))
-	     (let* ((arglist nil)
+	     (let* ((math-arglist nil)
 		    (has-args nil)
 		    (record-entry nil)
 		    (expr (if (eq key ?\$)
@@ -592,13 +591,13 @@
 			      (if (> calc-dollar-used 0)
 				  (progn
 				    (setq has-args calc-dollar-used
-					  arglist (calc-invent-args has-args))
+					  math-arglist (calc-invent-args has-args))
 				    (math-multi-subst (car func)
-						      (reverse arglist)
-						      arglist))
+						      (reverse math-arglist)
+						      math-arglist))
 				(if (> calc-hashes-used 0)
 				    (setq has-args calc-hashes-used
-					  arglist (calc-invent-args has-args)))
+					  math-arglist (calc-invent-args has-args)))
 				(car func))))))
 	       (if (eq (car-safe expr) 'calcFunc-lambda)
 		   (setq oper (list "$" (- (length expr) 2) expr)
@@ -607,16 +606,16 @@
 		     (progn
 		       (calc-default-formula-arglist expr)
 		       (setq record-entry t
-			     arglist (sort arglist 'string-lessp))
+			     math-arglist (sort math-arglist 'string-lessp))
 		       (if calc-verify-arglist
-			   (setq arglist (read-from-minibuffer
+			   (setq math-arglist (read-from-minibuffer
 					  "Function argument list: "
-					  (if arglist
-					      (prin1-to-string arglist)
+					  (if math-arglist
+					      (prin1-to-string math-arglist)
 					    "()")
 					  minibuffer-local-map
 					  t)))
-		       (setq arglist (mapcar (function
+		       (setq math-arglist (mapcar (function
 					      (lambda (x)
 						(list 'var
 						      x
@@ -624,10 +623,10 @@
 						       (concat
 							"var-"
 							(symbol-name x))))))
-					     arglist))))
+					     math-arglist))))
 		 (setq oper (list "$"
-				  (length arglist)
-				  (append '(calcFunc-lambda) arglist
+				  (length math-arglist)
+				  (append '(calcFunc-lambda) math-arglist
 					  (list expr)))
 		       done t))
 	       (if record-entry
@@ -1274,5 +1273,4 @@
 
 (provide 'calc-map)
 
-;; arch-tag: 980eac49-00e0-4870-b72a-e726b74c7990
 ;;; calc-map.el ends here
