@@ -1,7 +1,6 @@
 ;;; jka-compr.el --- reading/writing/loading compressed files
 
-;; Copyright (C) 1993, 1994, 1995, 1997, 1999, 2000, 2001, 2002, 2003,
-;;   2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+;; Copyright (C) 1993-1995, 1997, 1999-2012 Free Software Foundation, Inc.
 
 ;; Author: jka@ece.cmu.edu (Jay K. Adams)
 ;; Maintainer: FSF
@@ -184,7 +183,8 @@ to keep: LEN chars starting BEG chars from the beginning."
 			  null-device))
 			jka-compr-acceptable-retval-list)
 		  (jka-compr-error prog args infile message err-file))
-	    (jka-compr-delete-temp-file err-file)))
+	    (delete-file err-file)))
+
       ;; Run the uncompression program directly.
       ;; We get the whole file and must delete what we don't want.
       (jka-compr-call-process prog message infile t nil args))
@@ -203,6 +203,7 @@ to keep: LEN chars starting BEG chars from the beginning."
   ;; call-process barfs if default-directory is inaccessible.
   (let ((default-directory
 	  (if (and default-directory
+		   (not (file-remote-p default-directory))
 		   (file-accessible-directory-p default-directory))
 	      default-directory
 	    (file-name-directory infile))))
@@ -225,7 +226,7 @@ to keep: LEN chars starting BEG chars from the beginning."
 					   "")))
 		   jka-compr-acceptable-retval-list)
 		  (jka-compr-error prog args infile message err-file))
-	    (jka-compr-delete-temp-file err-file)))
+	    (delete-file err-file)))
       (or (eq 0
 	      (apply 'call-process
 		     prog infile (if (stringp output) temp output)
@@ -247,13 +248,10 @@ There should be no more than seven characters after the final `/'."
   :type 'string
   :group 'jka-compr)
 
-(defun jka-compr-make-temp-name (&optional local-copy)
+(defun jka-compr-make-temp-name (&optional _local-copy)
   "This routine will return the name of a new file."
   (make-temp-file jka-compr-temp-name-template))
 
-(defalias 'jka-compr-delete-temp-file 'delete-file)
-
-
 (defun jka-compr-write-region (start end file &optional append visit)
   (let* ((filename (expand-file-name file))
 	 (visit-file (if (stringp visit) (expand-file-name visit) filename))
@@ -312,6 +310,7 @@ There should be no more than seven characters after the final `/'."
 
 	  (and
 	   compress-message
+	   jka-compr-verbose
 	   (message "%s %s..." compress-message base-name))
 
 	  (jka-compr-run-real-handler 'write-region
@@ -340,10 +339,11 @@ There should be no more than seven characters after the final `/'."
 						(and append can-append) 'dont))
 	      (erase-buffer)) )
 
-	  (jka-compr-delete-temp-file temp-file)
+	  (delete-file temp-file)
 
 	  (and
 	   compress-message
+	   jka-compr-verbose
 	   (message "%s %s...done" compress-message base-name))
 
 	  (cond
@@ -407,6 +407,7 @@ There should be no more than seven characters after the final `/'."
 
               (and
                uncompress-message
+	       jka-compr-verbose
                (message "%s %s..." uncompress-message base-name))
 
               (condition-case error-code
@@ -482,6 +483,7 @@ There should be no more than seven characters after the final `/'."
 
         (and
          uncompress-message
+	 jka-compr-verbose
          (message "%s %s...done" uncompress-message base-name))
 
         (and
@@ -537,6 +539,7 @@ There should be no more than seven characters after the final `/'."
 
 		(and
 		 uncompress-message
+		 jka-compr-verbose
 		 (message "%s %s..." uncompress-message base-name))
 
 		;; Here we must read the output of uncompress program
@@ -557,6 +560,7 @@ There should be no more than seven characters after the final `/'."
 
 		  (and
 		   uncompress-message
+		   jka-compr-verbose
 		   (message "%s %s...done" uncompress-message base-name))
 
 		  (write-region
@@ -575,7 +579,7 @@ There should be no more than seven characters after the final `/'."
 
 
 ;; Support for loading compressed files.
-(defun jka-compr-load (file &optional noerror nomessage nosuffix)
+(defun jka-compr-load (file &optional noerror nomessage _nosuffix)
   "Documented as original."
 
   (let* ((local-copy (jka-compr-file-local-copy file))
@@ -606,7 +610,7 @@ There should be no more than seven characters after the final `/'."
 	      (setq file (file-name-sans-extension file)))
 	    (setcar l file)))
 
-      (jka-compr-delete-temp-file local-copy))
+      (delete-file local-copy))
 
     t))
 
@@ -654,16 +658,15 @@ It is not recommended to set this variable permanently to anything but nil.")
 (defun jka-compr-uninstall ()
   "Uninstall jka-compr.
 This removes the entries in `file-name-handler-alist' and `auto-mode-alist'
-and `inhibit-first-line-modes-suffixes' that were added
+and `inhibit-local-variables-suffixes' that were added
 by `jka-compr-installed'."
-  ;; Delete from inhibit-first-line-modes-suffixes
-  ;; what jka-compr-install added.
+  ;; Delete from inhibit-local-variables-suffixes what jka-compr-install added.
   (mapc
      (function (lambda (x)
 		 (and (jka-compr-info-strip-extension x)
-		      (setq inhibit-first-line-modes-suffixes
+		      (setq inhibit-local-variables-suffixes
 			    (delete (jka-compr-info-regexp x)
-				    inhibit-first-line-modes-suffixes)))))
+				    inhibit-local-variables-suffixes)))))
      jka-compr-compression-info-list--internal)
 
   (let* ((fnha (cons nil file-name-handler-alist))
@@ -706,5 +709,4 @@ by `jka-compr-installed'."
 
 (provide 'jka-compr)
 
-;; arch-tag: 3f15b630-e9a7-46c4-a22a-94afdde86ebc
 ;;; jka-compr.el ends here
