@@ -1,6 +1,5 @@
-/* Pro-process emacs.exe for profiling by MSVC.
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-     2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+/* Pre-process emacs.exe for profiling by MSVC.
+   Copyright (C) 1999, 2001-2012  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -37,9 +36,9 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 PIMAGE_NT_HEADERS
 (__stdcall * pfnCheckSumMappedFile) (LPVOID BaseAddress,
-				    DWORD FileLength,
-				    LPDWORD HeaderSum,
-				    LPDWORD CheckSum);
+				     DWORD FileLength,
+				     LPDWORD HeaderSum,
+				     LPDWORD CheckSum);
 
 #undef min
 #undef max
@@ -50,15 +49,15 @@ PIMAGE_NT_HEADERS
 /* File handling.  */
 
 typedef struct file_data {
-    char          *name;
-    unsigned long  size;
-    HANDLE         file;
-    HANDLE         file_mapping;
-    unsigned char *file_base;
+  const char    *name;
+  unsigned long  size;
+  HANDLE         file;
+  HANDLE         file_mapping;
+  unsigned char *file_base;
 } file_data;
 
 int
-open_input_file (file_data *p_file, char *filename)
+open_input_file (file_data *p_file, const char *filename)
 {
   HANDLE file;
   HANDLE file_mapping;
@@ -90,7 +89,7 @@ open_input_file (file_data *p_file, char *filename)
 }
 
 int
-open_output_file (file_data *p_file, char *filename, unsigned long size)
+open_output_file (file_data *p_file, const char *filename, unsigned long size)
 {
   HANDLE file;
   HANDLE file_mapping;
@@ -120,7 +119,7 @@ open_output_file (file_data *p_file, char *filename, unsigned long size)
 }
 
 int
-open_inout_file (file_data *p_file, char *filename)
+open_inout_file (file_data *p_file, const char *filename)
 {
   HANDLE file;
   HANDLE file_mapping;
@@ -178,7 +177,7 @@ get_unrounded_section_size (PIMAGE_SECTION_HEADER p_section)
 
 /* Return pointer to section header for named section. */
 IMAGE_SECTION_HEADER *
-find_section (char * name, IMAGE_NT_HEADERS * nt_header)
+find_section (const char *name, IMAGE_NT_HEADERS *nt_header)
 {
   PIMAGE_SECTION_HEADER section;
   int i;
@@ -295,7 +294,7 @@ relocate_offset (DWORD offset,
 #define PTR_TO_RVA(ptr) ((DWORD)(ptr) - (DWORD) GetModuleHandle (NULL))
 
 #define PTR_TO_OFFSET(ptr, pfile_data) \
-          ((unsigned char *)(ptr) - (pfile_data)->file_base)
+          ((unsigned const char *)(ptr) - (pfile_data)->file_base)
 
 #define OFFSET_TO_PTR(offset, pfile_data) \
           ((pfile_data)->file_base + (DWORD)(offset))
@@ -322,7 +321,7 @@ relocate_offset (DWORD offset,
    variables, because of dumping.]
 
    We could potentially generate the relocation data ourselves by making
-   two versions of temacs, one with an extra dummmy section before
+   two versions of temacs, one with an extra dummy section before
    EMHEAP to offset it, and then compare the dumped executables from
    both.  That is a lot of work though, and it doesn't solve the problem
    of dumped pointers to static variables, which also can be relocated.
@@ -361,7 +360,7 @@ copy_executable_and_move_sections (file_data *p_infile,
 
 #define COPY_CHUNK(message, src, size)						\
   do {										\
-    unsigned char *s = (void *)(src);						\
+    unsigned const char *s = (void *)(src);					\
     unsigned long count = (size);						\
     printf ("%s\n", (message));							\
     printf ("\t0x%08x Offset in input file.\n", s - p_infile->file_base);	\
@@ -766,7 +765,6 @@ main (int argc, char **argv)
   PIMAGE_NT_HEADERS nt_header;
   file_data in_file, out_file;
   char out_filename[MAX_PATH], in_filename[MAX_PATH];
-  char *ptr;
 
   strcpy (in_filename, argv[1]);
   strcpy (out_filename, argv[2]);
@@ -827,6 +825,3 @@ main (int argc, char **argv)
 }
 
 /* eof */
-
-/* arch-tag: 144ca747-168e-43a0-9736-3f4c0ba1657f
-   (do not change this comment) */
