@@ -7,30 +7,13 @@
  * copyright, constituting the only possible expression of the
  * algorithm in this format.
  */
-#ifdef VMS
-#ifndef FAB$C_BID
-#include <fab.h>
-#endif
-#ifndef NAM$C_BID
-#include <nam.h>
-#endif
-#ifndef RMS$_SUC
-#include <rmsdef.h>
-#endif
-#include "vmsdir.h"
-#endif /* VMS */
 
 #define DIRBLKSIZ	512		/* size of directory block */
-#ifdef VMS
-#define MAXNAMLEN	(DIR$S_NAME + 7) /* 80 plus room for version #.  */
-#define MAXFULLSPEC	NAM$C_MAXRSS /* Maximum full spec */
-#else
 #ifdef WINDOWSNT
 #define MAXNAMLEN	255
 #else  /* not WINDOWSNT */
 #define MAXNAMLEN	15		/* maximum filename length */
 #endif /* not WINDOWSNT */
-#endif /* VMS */
 	/* NOTE:  MAXNAMLEN must be one less than a multiple of 4 */
 
 struct direct				/* data from readdir() */
@@ -49,13 +32,10 @@ typedef struct
 	char	dd_buf[DIRBLKSIZ];	/* directory block */
 	}	DIR;			/* stream data from opendir() */
 
-extern DIR		*opendir();
-extern struct direct	*readdir();
-extern long		telldir();
-extern void		seekdir();
-extern void		closedir();
+extern DIR		*opendir (char *);
+extern struct direct	*readdir (DIR *);
+extern void		seekdir (DIR *, long);
+extern void		closedir (DIR *);
 
 #define rewinddir( dirp )	seekdir( dirp, 0L )
 
-/* arch-tag: aea50570-ffb7-43fd-b423-7743b10fbe6e
-   (do not change this comment) */
