@@ -1,6 +1,5 @@
 /* Flags and parameters describing user options for handling the terminal.
-   Copyright (C) 1985, 1986, 1990, 2001, 2002, 2003, 2004,
-                 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+   Copyright (C) 1985-1986, 1990, 2001-2012  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -18,12 +17,6 @@ You should have received a copy of the GNU General Public License
 along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
-/* Nonzero means flash the screen instead of ringing the bell.  */
-extern int visible_bell;
-
-/* Nonzero means invert white and black for the entire screen.  */
-extern int inverse_video;
-
 /* Nonzero means use ^S/^Q as cretinous flow control.  */
 extern int flow_control;
 
@@ -36,13 +29,3 @@ extern int interrupts_deferred;
 /* Terminal has meta key */
 extern int meta_key;
 
-/* Defined in xdisp.c */
-extern Lisp_Object Vtruncate_partial_width_windows;
-
-/* Nonzero means no need to redraw the entire frame on resuming a suspended
-   Emacs.  This is useful on terminals with multiple pages, where one page is
-   used for Emacs and another for all else. */
-extern int no_redraw_on_reenter;
-
-/* arch-tag: 35d4d284-dc1a-4fff-97fa-0154a21aebdb
-   (do not change this comment) */
